/*_############################################################################
  _##
  _##  SNMP4J-Agent 2 - BasicVacmConfigurator.java
  _##
  _##  Copyright (C) 2005-2017  Frank Fock (SNMP4J.org)
  _##
  _##  Licensed under the Apache License, Version 2.0 (the "License");
  _##  you may not use this file except in compliance with the License.
  _##  You may obtain a copy of the License at
  _##
  _##      http://www.apache.org/licenses/LICENSE-2.0
  _##
  _##  Unless required by applicable law or agreed to in writing, software
  _##  distributed under the License is distributed on an "AS IS" BASIS,
  _##  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  _##  See the License for the specific language governing permissions and
  _##  limitations under the License.
  _##
  _##########################################################################*/

package org.snmp4j.agent.security;

import org.snmp4j.agent.mo.snmp.StorageType;
import org.snmp4j.agent.mo.snmp.VacmMIB;
import org.snmp4j.security.SecurityModel;
import org.snmp4j.smi.OID;
import org.snmp4j.smi.OctetString;

import java.util.Set;

/**
 * The {@code BasicVacmConfigurator} implements the {@link VacmConfigurator} that uses three unrestricted views
 * for all access types to support two roles {@code admin} and {@code monitor}. The {@code admin} role has unrestricted
 * access to the agent, whereas {@code monitor} has no write access, but unrestricted read and notification access.
 */
public class BasicVacmConfigurator  implements VacmConfigurator {

    protected OctetString rootViewName = new OctetString("rootView");

    protected OID rootOID = new OID(new int[] { 1 });

    @Override
    public VacmConfigResult addUser(MutableVACM vacm, OctetString securityName, String role) {
        OctetString groupName = new OctetString(role);
        if (vacm.hasSecurityToGroupMapping(SecurityModel.SECURITY_MODEL_USM, securityName) ||
                vacm.hasSecurityToGroupMapping(SecurityModel.SECURITY_MODEL_TSM, securityName)) {
            return VacmConfigResult.userExists;
        }
        if ((vacm.accessEntryCount(groupName) > 0) && (vacm.viewTreeFamilyEntryCount(rootViewName) > 0)) {
            return VacmConfigResult.viewExists;
        }
        vacm.addViewTreeFamily(rootViewName, rootOID, new OctetString(), VacmMIB.VACM_VIEW_INCLUDED,
                StorageType.permanent);
        return VacmConfigResult.userAndRoleAdded;
    }

    @Override
    public VacmConfigResult removeUser(MutableVACM vacm, OctetString securityName, String role) {
        return null;
    }

    @Override
    public VacmConfigResult removeRole(MutableVACM vacm, String role) {
        return null;
    }

    @Override
    public String[] getSupportedRoles() {
        return new String[] { "admin", "monitor" };
    }
}
